import { bytes, uint64, biguint } from './primitives';
import { Account, Application, Asset } from './reference';
import { OnCompleteAction } from './on-complete-action';
import { TransactionType } from './transactions';
export declare enum Base64 {
    URLEncoding = "URLEncoding",
    StdEncoding = "StdEncoding"
}
export declare enum Ec {
    /**
     * G1 of the BN254 curve. Points encoded as 32 byte X following by 32 byte Y
     */
    BN254g1 = "BN254g1",
    /**
     * G2 of the BN254 curve. Points encoded as 64 byte X following by 64 byte Y
     */
    BN254g2 = "BN254g2",
    /**
     * G1 of the BLS 12-381 curve. Points encoded as 48 byte X following by 48 byte Y
     */
    BLS12_381g1 = "BLS12_381g1",
    /**
     * G2 of the BLS 12-381 curve. Points encoded as 96 byte X following by 96 byte Y
     */
    BLS12_381g2 = "BLS12_381g2"
}
export declare enum Ecdsa {
    /**
     * secp256k1 curve, used in Bitcoin
     */
    Secp256k1 = "Secp256k1",
    /**
     * secp256r1 curve, NIST standard
     */
    Secp256r1 = "Secp256r1"
}
export declare enum MimcConfigurations {
    /**
     * MiMC configuration for the BN254 curve with Miyaguchi-Preneel mode, 110 rounds, exponent 5, seed "seed"
     */
    BN254Mp110 = "BN254Mp110",
    /**
     * MiMC configuration for the BLS12-381 curve with Miyaguchi-Preneel mode, 111 rounds, exponent 5, seed "seed"
     */
    BLS12_381Mp111 = "BLS12_381Mp111"
}
export declare enum VrfVerify {
    VrfAlgorand = "VrfAlgorand"
}
export declare const AcctParams: {
    /**
     * Account balance in microalgos
     * Min AVM version: 6
     */
    acctBalance(a: Account | uint64): readonly [uint64, boolean];
    /**
     * Minimum required balance for account, in microalgos
     * Min AVM version: 6
     */
    acctMinBalance(a: Account | uint64): readonly [uint64, boolean];
    /**
     * Address the account is rekeyed to.
     * Min AVM version: 6
     */
    acctAuthAddr(a: Account | uint64): readonly [Account, boolean];
    /**
     * The total number of uint64 values allocated by this account in Global and Local States.
     * Min AVM version: 8
     */
    acctTotalNumUint(a: Account | uint64): readonly [uint64, boolean];
    /**
     * The total number of byte array values allocated by this account in Global and Local States.
     * Min AVM version: 8
     */
    acctTotalNumByteSlice(a: Account | uint64): readonly [uint64, boolean];
    /**
     * The number of extra app code pages used by this account.
     * Min AVM version: 8
     */
    acctTotalExtraAppPages(a: Account | uint64): readonly [uint64, boolean];
    /**
     * The number of existing apps created by this account.
     * Min AVM version: 8
     */
    acctTotalAppsCreated(a: Account | uint64): readonly [uint64, boolean];
    /**
     * The number of apps this account is opted into.
     * Min AVM version: 8
     */
    acctTotalAppsOptedIn(a: Account | uint64): readonly [uint64, boolean];
    /**
     * The number of existing ASAs created by this account.
     * Min AVM version: 8
     */
    acctTotalAssetsCreated(a: Account | uint64): readonly [uint64, boolean];
    /**
     * The numbers of ASAs held by this account (including ASAs this account created).
     * Min AVM version: 8
     */
    acctTotalAssets(a: Account | uint64): readonly [uint64, boolean];
    /**
     * The number of existing boxes created by this account's app.
     * Min AVM version: 8
     */
    acctTotalBoxes(a: Account | uint64): readonly [uint64, boolean];
    /**
     * The total number of bytes used by this account's app's box keys and values.
     * Min AVM version: 8
     */
    acctTotalBoxBytes(a: Account | uint64): readonly [uint64, boolean];
    /**
     * Has this account opted into block payouts
     * Min AVM version: 11
     */
    acctIncentiveEligible(a: Account | uint64): readonly [boolean, boolean];
    /**
     * The round number of the last block this account proposed.
     * Min AVM version: 11
     */
    acctLastProposed(a: Account | uint64): readonly [uint64, boolean];
    /**
     * The round number of the last block this account sent a heartbeat.
     * Min AVM version: 11
     */
    acctLastHeartbeat(a: Account | uint64): readonly [uint64, boolean];
};
/**
 * A plus B as a 128-bit result. X is the carry-bit, Y is the low-order 64 bits.
 * @see Native TEAL opcode: [`addw`](https://dev.algorand.co/reference/algorand-teal/opcodes#addw)
 * Min AVM version: 2
 */
export declare function addw(a: uint64, b: uint64): readonly [uint64, uint64];
/**
 * Get or modify Global app state
 */
export declare const AppGlobal: {
    /**
     * delete key A from the global state of the current application
     * @param state key.
     * Deleting a key which is already absent has no effect on the application global state. (In particular, it does _not_ cause the program to fail.)
     * @see Native TEAL opcode: [`app_global_del`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_global_del)
     * Min AVM version: 2
     */
    delete(a: bytes): void;
    /**
     * global state of the key A in the current application
     * @param state key.
     *  * @return value. The value is zero (of type uint64) if the key does not exist.
     * @see Native TEAL opcode: [`app_global_get`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_global_get)
     * Min AVM version: 2
     */
    getBytes(a: bytes): bytes;
    /**
     * global state of the key A in the current application
     * @param state key.
     *  * @return value. The value is zero (of type uint64) if the key does not exist.
     * @see Native TEAL opcode: [`app_global_get`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_global_get)
     * Min AVM version: 2
     */
    getUint64(a: bytes): uint64;
    /**
     * X is the global state of application A, key B. Y is 1 if key existed, else 0
     * @param Txn.ForeignApps offset (or, since v4, an _available_ application id), state key.
     *  * @return did_exist flag (top of the stack, 1 if the application and key existed and 0 otherwise), value. The value is zero (of type uint64) if the key does not exist.
     * @see Native TEAL opcode: [`app_global_get_ex`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_global_get_ex)
     * Min AVM version: 2
     */
    getExBytes(a: Application | uint64, b: bytes): readonly [bytes, boolean];
    /**
     * X is the global state of application A, key B. Y is 1 if key existed, else 0
     * @param Txn.ForeignApps offset (or, since v4, an _available_ application id), state key.
     *  * @return did_exist flag (top of the stack, 1 if the application and key existed and 0 otherwise), value. The value is zero (of type uint64) if the key does not exist.
     * @see Native TEAL opcode: [`app_global_get_ex`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_global_get_ex)
     * Min AVM version: 2
     */
    getExUint64(a: Application | uint64, b: bytes): readonly [uint64, boolean];
    /**
     * write B to key A in the global state of the current application
     * @see Native TEAL opcode: [`app_global_put`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_global_put)
     * Min AVM version: 2
     */
    put(a: bytes, b: uint64 | bytes): void;
};
/**
 * Get or modify Local app state
 */
export declare const AppLocal: {
    /**
     * delete key B from account A's local state of the current application
     * @param Txn.Accounts offset (or, since v4, an _available_ account address), state key.
     * Deleting a key which is already absent has no effect on the application local state. (In particular, it does _not_ cause the program to fail.)
     * @see Native TEAL opcode: [`app_local_del`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_local_del)
     * Min AVM version: 2
     */
    delete(a: Account | uint64, b: bytes): void;
    /**
     * local state of the key B in the current application in account A
     * @param Txn.Accounts offset (or, since v4, an _available_ account address), state key.
     *  * @return value. The value is zero (of type uint64) if the key does not exist.
     * @see Native TEAL opcode: [`app_local_get`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_local_get)
     * Min AVM version: 2
     */
    getBytes(a: Account | uint64, b: bytes): bytes;
    /**
     * local state of the key B in the current application in account A
     * @param Txn.Accounts offset (or, since v4, an _available_ account address), state key.
     *  * @return value. The value is zero (of type uint64) if the key does not exist.
     * @see Native TEAL opcode: [`app_local_get`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_local_get)
     * Min AVM version: 2
     */
    getUint64(a: Account | uint64, b: bytes): uint64;
    /**
     * X is the local state of application B, key C in account A. Y is 1 if key existed, else 0
     * @param Txn.Accounts offset (or, since v4, an _available_ account address), _available_ application id (or, since v4, a Txn.ForeignApps offset), state key.
     *  * @return did_exist flag (top of the stack, 1 if the application and key existed and 0 otherwise), value. The value is zero (of type uint64) if the key does not exist.
     * @see Native TEAL opcode: [`app_local_get_ex`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_local_get_ex)
     * Min AVM version: 2
     */
    getExBytes(a: Account | uint64, b: Application | uint64, c: bytes): readonly [bytes, boolean];
    /**
     * X is the local state of application B, key C in account A. Y is 1 if key existed, else 0
     * @param Txn.Accounts offset (or, since v4, an _available_ account address), _available_ application id (or, since v4, a Txn.ForeignApps offset), state key.
     *  * @return did_exist flag (top of the stack, 1 if the application and key existed and 0 otherwise), value. The value is zero (of type uint64) if the key does not exist.
     * @see Native TEAL opcode: [`app_local_get_ex`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_local_get_ex)
     * Min AVM version: 2
     */
    getExUint64(a: Account | uint64, b: Application | uint64, c: bytes): readonly [uint64, boolean];
    /**
     * write C to key B in account A's local state of the current application
     * @param Txn.Accounts offset (or, since v4, an _available_ account address), state key, value.
     * @see Native TEAL opcode: [`app_local_put`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_local_put)
     * Min AVM version: 2
     */
    put(a: Account | uint64, b: bytes, c: uint64 | bytes): void;
};
/**
 * 1 if account A is opted in to application B, else 0
 * @param Txn.Accounts offset (or, since v4, an _available_ account address), _available_ application id (or, since v4, a Txn.ForeignApps offset).
 *  * @return 1 if opted in and 0 otherwise.
 * @see Native TEAL opcode: [`app_opted_in`](https://dev.algorand.co/reference/algorand-teal/opcodes#app_opted_in)
 * Min AVM version: 2
 */
export declare function appOptedIn(a: Account | uint64, b: Application | uint64): boolean;
export declare const AppParams: {
    /**
     * Bytecode of Approval Program
     * Min AVM version: 5
     */
    appApprovalProgram(a: Application | uint64): readonly [bytes, boolean];
    /**
     * Bytecode of Clear State Program
     * Min AVM version: 5
     */
    appClearStateProgram(a: Application | uint64): readonly [bytes, boolean];
    /**
     * Number of uint64 values allowed in Global State
     * Min AVM version: 5
     */
    appGlobalNumUint(a: Application | uint64): readonly [uint64, boolean];
    /**
     * Number of byte array values allowed in Global State
     * Min AVM version: 5
     */
    appGlobalNumByteSlice(a: Application | uint64): readonly [uint64, boolean];
    /**
     * Number of uint64 values allowed in Local State
     * Min AVM version: 5
     */
    appLocalNumUint(a: Application | uint64): readonly [uint64, boolean];
    /**
     * Number of byte array values allowed in Local State
     * Min AVM version: 5
     */
    appLocalNumByteSlice(a: Application | uint64): readonly [uint64, boolean];
    /**
     * Number of Extra Program Pages of code space
     * Min AVM version: 5
     */
    appExtraProgramPages(a: Application | uint64): readonly [uint64, boolean];
    /**
     * Creator address
     * Min AVM version: 5
     */
    appCreator(a: Application | uint64): readonly [Account, boolean];
    /**
     * Address for which this application has authority
     * Min AVM version: 5
     */
    appAddress(a: Application | uint64): readonly [Account, boolean];
};
/**
 * Ath LogicSig argument
 * @see Native TEAL opcode: [`args`](https://dev.algorand.co/reference/algorand-teal/opcodes#args)
 * Min AVM version: 5
 */
export declare function arg(a: uint64): bytes;
export declare const AssetHolding: {
    /**
     * Amount of the asset unit held by this account
     * Min AVM version: 2
     */
    assetBalance(a: Account | uint64, b: Asset | uint64): readonly [uint64, boolean];
    /**
     * Is the asset frozen or not
     * Min AVM version: 2
     */
    assetFrozen(a: Account | uint64, b: Asset | uint64): readonly [boolean, boolean];
};
export declare const AssetParams: {
    /**
     * Total number of units of this asset
     * Min AVM version: 2
     */
    assetTotal(a: Asset | uint64): readonly [uint64, boolean];
    /**
     * See AssetParams.Decimals
     * Min AVM version: 2
     */
    assetDecimals(a: Asset | uint64): readonly [uint64, boolean];
    /**
     * Frozen by default or not
     * Min AVM version: 2
     */
    assetDefaultFrozen(a: Asset | uint64): readonly [boolean, boolean];
    /**
     * Asset unit name
     * Min AVM version: 2
     */
    assetUnitName(a: Asset | uint64): readonly [bytes, boolean];
    /**
     * Asset name
     * Min AVM version: 2
     */
    assetName(a: Asset | uint64): readonly [bytes, boolean];
    /**
     * URL with additional info about the asset
     * Min AVM version: 2
     */
    assetUrl(a: Asset | uint64): readonly [bytes, boolean];
    /**
     * Arbitrary commitment
     * Min AVM version: 2
     */
    assetMetadataHash(a: Asset | uint64): readonly [bytes, boolean];
    /**
     * Manager address
     * Min AVM version: 2
     */
    assetManager(a: Asset | uint64): readonly [Account, boolean];
    /**
     * Reserve address
     * Min AVM version: 2
     */
    assetReserve(a: Asset | uint64): readonly [Account, boolean];
    /**
     * Freeze address
     * Min AVM version: 2
     */
    assetFreeze(a: Asset | uint64): readonly [Account, boolean];
    /**
     * Clawback address
     * Min AVM version: 2
     */
    assetClawback(a: Asset | uint64): readonly [Account, boolean];
    /**
     * Creator address
     * Min AVM version: 5
     */
    assetCreator(a: Asset | uint64): readonly [Account, boolean];
};
/**
 * balance for account A, in microalgos. The balance is observed after the effects of previous transactions in the group, and after the fee for the current transaction is deducted. Changes caused by inner transactions are observable immediately following `itxn_submit`
 * @param Txn.Accounts offset (or, since v4, an _available_ account address), _available_ application id (or, since v4, a Txn.ForeignApps offset).
 *  * @return value.
 * @see Native TEAL opcode: [`balance`](https://dev.algorand.co/reference/algorand-teal/opcodes#balance)
 * Min AVM version: 2
 */
export declare function balance(a: Account | uint64): uint64;
/**
 * decode A which was base64-encoded using _encoding_ E. Fail if A is not base64 encoded with encoding E
 * *Warning*: Usage should be restricted to very rare use cases. In almost all cases, smart contracts should directly handle non-encoded byte-strings.	This opcode should only be used in cases where base64 is the only available option, e.g. interoperability with a third-party that only signs base64 strings.
 *  Decodes A using the base64 encoding E. Specify the encoding with an immediate arg either as URL and Filename Safe (`URLEncoding`) or Standard (`StdEncoding`). See [RFC 4648 sections 4 and 5](https://rfc-editor.org/rfc/rfc4648.html#section-4). It is assumed that the encoding ends with the exact number of `=` padding characters as required by the RFC. When padding occurs, any unused pad bits in the encoding must be set to zero or the decoding will fail. The special cases of `\n` and `\r` are allowed but completely ignored. An error will result when attempting to decode a string with a character that is not in the encoding alphabet or not one of `=`, `\r`, or `\n`.
 * @see Native TEAL opcode: [`base64_decode`](https://dev.algorand.co/reference/algorand-teal/opcodes#base64_decode)
 * Min AVM version: 7
 */
export declare function base64Decode(e: Base64, a: bytes): bytes;
/**
 * The highest set bit in A. If A is a byte-array, it is interpreted as a big-endian unsigned integer. bitlen of 0 is 0, bitlen of 8 is 4
 * bitlen interprets arrays as big-endian integers, unlike setbit/getbit
 * @see Native TEAL opcode: [`bitlen`](https://dev.algorand.co/reference/algorand-teal/opcodes#bitlen)
 * Min AVM version: 4
 */
export declare function bitLength(a: uint64 | bytes): uint64;
export declare const Block: {
    blkSeed(a: uint64): bytes;
    blkTimestamp(a: uint64): uint64;
    blkProposer(a: uint64): Account;
    blkFeesCollected(a: uint64): uint64;
    blkBonus(a: uint64): uint64;
    blkBranch(a: uint64): bytes;
    blkFeeSink(a: uint64): Account;
    blkProtocol(a: uint64): bytes;
    blkTxnCounter(a: uint64): uint64;
    blkProposerPayout(a: uint64): uint64;
};
/**
 * Get or modify box state
 */
export declare const Box: {
    /**
     * create a box named A, of length B. Fail if the name A is empty or B exceeds 32,768. Returns 0 if A already existed, else 1
     * Newly created boxes are filled with 0 bytes. `box_create` will fail if the referenced box already exists with a different size. Otherwise, existing boxes are unchanged by `box_create`.
     * @see Native TEAL opcode: [`box_create`](https://dev.algorand.co/reference/algorand-teal/opcodes#box_create)
     * Min AVM version: 8
     */
    create(a: bytes, b: uint64): boolean;
    /**
     * delete box named A if it exists. Return 1 if A existed, 0 otherwise
     * @see Native TEAL opcode: [`box_del`](https://dev.algorand.co/reference/algorand-teal/opcodes#box_del)
     * Min AVM version: 8
     */
    delete(a: bytes): boolean;
    /**
     * read C bytes from box A, starting at offset B. Fail if A does not exist, or the byte range is outside A's size.
     * @see Native TEAL opcode: [`box_extract`](https://dev.algorand.co/reference/algorand-teal/opcodes#box_extract)
     * Min AVM version: 8
     */
    extract(a: bytes, b: uint64, c: uint64): bytes;
    /**
     * X is the contents of box A if A exists, else ''. Y is 1 if A exists, else 0.
     * For boxes that exceed 4,096 bytes, consider `box_create`, `box_extract`, and `box_replace`
     * @see Native TEAL opcode: [`box_get`](https://dev.algorand.co/reference/algorand-teal/opcodes#box_get)
     * Min AVM version: 8
     */
    get(a: bytes): readonly [bytes, boolean];
    /**
     * X is the length of box A if A exists, else 0. Y is 1 if A exists, else 0.
     * @see Native TEAL opcode: [`box_len`](https://dev.algorand.co/reference/algorand-teal/opcodes#box_len)
     * Min AVM version: 8
     */
    length(a: bytes): readonly [uint64, boolean];
    /**
     * replaces the contents of box A with byte-array B. Fails if A exists and len(B) != len(box A). Creates A if it does not exist
     * For boxes that exceed 4,096 bytes, consider `box_create`, `box_extract`, and `box_replace`
     * @see Native TEAL opcode: [`box_put`](https://dev.algorand.co/reference/algorand-teal/opcodes#box_put)
     * Min AVM version: 8
     */
    put(a: bytes, b: bytes): void;
    /**
     * write byte-array C into box A, starting at offset B. Fail if A does not exist, or the byte range is outside A's size.
     * @see Native TEAL opcode: [`box_replace`](https://dev.algorand.co/reference/algorand-teal/opcodes#box_replace)
     * Min AVM version: 8
     */
    replace(a: bytes, b: uint64, c: bytes): void;
    /**
     * change the size of box named A to be of length B, adding zero bytes to end or removing bytes from the end, as needed. Fail if the name A is empty, A is not an existing box, or B exceeds 32,768.
     * @see Native TEAL opcode: [`box_resize`](https://dev.algorand.co/reference/algorand-teal/opcodes#box_resize)
     * Min AVM version: 10
     */
    resize(a: bytes, b: uint64): void;
    /**
     * set box A to contain its previous bytes up to index B, followed by D, followed by the original bytes of A that began at index B+C.
     * Boxes are of constant length. If C < len(D), then len(D)-C bytes will be removed from the end. If C > len(D), zero bytes will be appended to the end to reach the box length.
     * @see Native TEAL opcode: [`box_splice`](https://dev.algorand.co/reference/algorand-teal/opcodes#box_splice)
     * Min AVM version: 10
     */
    splice(a: bytes, b: uint64, c: uint64, d: bytes): void;
};
/**
 * The largest integer I such that I^2 <= A. A and I are interpreted as big-endian unsigned integers
 * @see Native TEAL opcode: [`bsqrt`](https://dev.algorand.co/reference/algorand-teal/opcodes#bsqrt)
 * Min AVM version: 6
 */
export declare function bsqrt(a: biguint): biguint;
/**
 * converts big-endian byte array A to uint64. Fails if len(A) > 8. Padded by leading 0s if len(A) < 8.
 * `btoi` fails if the input is longer than 8 bytes.
 * @see Native TEAL opcode: [`btoi`](https://dev.algorand.co/reference/algorand-teal/opcodes#btoi)
 * Min AVM version: 1
 */
export declare function btoi(a: bytes): uint64;
/**
 * zero filled byte-array of length A
 * @see Native TEAL opcode: [`bzero`](https://dev.algorand.co/reference/algorand-teal/opcodes#bzero)
 * Min AVM version: 4
 */
export declare function bzero(a: uint64): bytes;
/**
 * join A and B
 * `concat` fails if the result would be greater than 4096 bytes.
 * @see Native TEAL opcode: [`concat`](https://dev.algorand.co/reference/algorand-teal/opcodes#concat)
 * Min AVM version: 2
 */
export declare function concat(a: bytes, b: bytes): bytes;
/**
 * W,X = (A,B / C,D); Y,Z = (A,B modulo C,D)
 * The notation J,K indicates that two uint64 values J and K are interpreted as a uint128 value, with J as the high uint64 and K the low.
 * @see Native TEAL opcode: [`divmodw`](https://dev.algorand.co/reference/algorand-teal/opcodes#divmodw)
 * Min AVM version: 4
 */
export declare function divmodw(a: uint64, b: uint64, c: uint64, d: uint64): readonly [uint64, uint64, uint64, uint64];
/**
 * A,B / C. Fail if C == 0 or if result overflows.
 * The notation A,B indicates that A and B are interpreted as a uint128 value, with A as the high uint64 and B the low.
 * @see Native TEAL opcode: [`divw`](https://dev.algorand.co/reference/algorand-teal/opcodes#divw)
 * Min AVM version: 6
 */
export declare function divw(a: uint64, b: uint64, c: uint64): uint64;
/**
 * Elliptic Curve functions
 */
export declare const EllipticCurve: {
    /**
     * for curve points A and B, return the curve point A + B
     * A and B are curve points in affine representation: field element X concatenated with field element Y. Field element `Z` is encoded as follows.
     * For the base field elements (Fp), `Z` is encoded as a big-endian number and must be lower than the field modulus.
     * For the quadratic field extension (Fp2), `Z` is encoded as the concatenation of the individual encoding of the coefficients. For an Fp2 element of the form `Z = Z0 + Z1 i`, where `i` is a formal quadratic non-residue, the encoding of Z is the concatenation of the encoding of `Z0` and `Z1` in this order. (`Z0` and `Z1` must be less than the field modulus).
     * The point at infinity is encoded as `(X,Y) = (0,0)`.
     * Groups G1 and G2 are denoted additively.
     * Fails if A or B is not in G.
     * A and/or B are allowed to be the point at infinity.
     * Does _not_ check if A and B are in the main prime-order subgroup.
     * @see Native TEAL opcode: [`ec_add`](https://dev.algorand.co/reference/algorand-teal/opcodes#ec_add)
     * Min AVM version: 10
     */
    add(g: Ec, a: bytes, b: bytes): bytes;
    /**
     * maps field element A to group G
     * BN254 points are mapped by the SVDW map. BLS12-381 points are mapped by the SSWU map.
     * G1 element inputs are base field elements and G2 element inputs are quadratic field elements, with nearly the same encoding rules (for field elements) as defined in `ec_add`. There is one difference of encoding rule: G1 element inputs do not need to be 0-padded if they fit in less than 32 bytes for BN254 and less than 48 bytes for BLS12-381. (As usual, the empty byte array represents 0.) G2 elements inputs need to be always have the required size.
     * @see Native TEAL opcode: [`ec_map_to`](https://dev.algorand.co/reference/algorand-teal/opcodes#ec_map_to)
     * Min AVM version: 10
     */
    mapTo(g: Ec, a: bytes): bytes;
    /**
     * for curve points A and scalars B, return curve point B0A0 + B1A1 + B2A2 + ... + BnAn
     * A is a list of concatenated points, encoded and checked as described in `ec_add`. B is a list of concatenated scalars which, unlike ec_scalar_mul, must all be exactly 32 bytes long.
     * The name `ec_multi_scalar_mul` was chosen to reflect common usage, but a more consistent name would be `ec_multi_scalar_mul`. AVM values are limited to 4096 bytes, so `ec_multi_scalar_mul` is limited by the size of the points in the group being operated upon.
     * @see Native TEAL opcode: [`ec_multi_scalar_mul`](https://dev.algorand.co/reference/algorand-teal/opcodes#ec_multi_scalar_mul)
     * Min AVM version: 10
     */
    scalarMulMulti(g: Ec, a: bytes, b: bytes): bytes;
    /**
     * 1 if the product of the pairing of each point in A with its respective point in B is equal to the identity element of the target group Gt, else 0
     * A and B are concatenated points, encoded and checked as described in `ec_add`. A contains points of the group G, B contains points of the associated group (G2 if G is G1, and vice versa). Fails if A and B have a different number of points, or if any point is not in its described group or outside the main prime-order subgroup - a stronger condition than other opcodes. AVM values are limited to 4096 bytes, so `ec_pairing_check` is limited by the size of the points in the groups being operated upon.
     * @see Native TEAL opcode: [`ec_pairing_check`](https://dev.algorand.co/reference/algorand-teal/opcodes#ec_pairing_check)
     * Min AVM version: 10
     */
    pairingCheck(g: Ec, a: bytes, b: bytes): boolean;
    /**
     * for curve point A and scalar B, return the curve point BA, the point A multiplied by the scalar B.
     * A is a curve point encoded and checked as described in `ec_add`. Scalar B is interpreted as a big-endian unsigned integer. Fails if B exceeds 32 bytes.
     * @see Native TEAL opcode: [`ec_scalar_mul`](https://dev.algorand.co/reference/algorand-teal/opcodes#ec_scalar_mul)
     * Min AVM version: 10
     */
    scalarMul(g: Ec, a: bytes, b: bytes): bytes;
    /**
     * 1 if A is in the main prime-order subgroup of G (including the point at infinity) else 0. Program fails if A is not in G at all.
     * @see Native TEAL opcode: [`ec_subgroup_check`](https://dev.algorand.co/reference/algorand-teal/opcodes#ec_subgroup_check)
     * Min AVM version: 10
     */
    subgroupCheck(g: Ec, a: bytes): boolean;
};
/**
 * decompress pubkey A into components X, Y
 * The 33 byte public key in a compressed form to be decompressed into X and Y (top) components. All values are big-endian encoded.
 * @see Native TEAL opcode: [`ecdsa_pk_decompress`](https://dev.algorand.co/reference/algorand-teal/opcodes#ecdsa_pk_decompress)
 * Min AVM version: 5
 */
export declare function ecdsaPkDecompress(v: Ecdsa, a: bytes): readonly [bytes, bytes];
/**
 * for (data A, recovery id B, signature C, D) recover a public key
 * S (top) and R elements of a signature, recovery id and data (bottom) are expected on the stack and used to deriver a public key. All values are big-endian encoded. The signed data must be 32 bytes long.
 * @see Native TEAL opcode: [`ecdsa_pk_recover`](https://dev.algorand.co/reference/algorand-teal/opcodes#ecdsa_pk_recover)
 * Min AVM version: 5
 */
export declare function ecdsaPkRecover(v: Ecdsa, a: bytes, b: uint64, c: bytes, d: bytes): readonly [bytes, bytes];
/**
 * for (data A, signature B, C and pubkey D, E) verify the signature of the data against the pubkey => {0 or 1}
 * The 32 byte Y-component of a public key is the last element on the stack, preceded by X-component of a pubkey, preceded by S and R components of a signature, preceded by the data that is fifth element on the stack. All values are big-endian encoded. The signed data must be 32 bytes long, and signatures in lower-S form are only accepted.
 * @see Native TEAL opcode: [`ecdsa_verify`](https://dev.algorand.co/reference/algorand-teal/opcodes#ecdsa_verify)
 * Min AVM version: 5
 */
export declare function ecdsaVerify(v: Ecdsa, a: bytes, b: bytes, c: bytes, d: bytes, e: bytes): boolean;
/**
 * for (data A, signature B, pubkey C) verify the signature of ("ProgData" || program_hash || data) against the pubkey => {0 or 1}
 * The 32 byte public key is the last element on the stack, preceded by the 64 byte signature at the second-to-last element on the stack, preceded by the data which was signed at the third-to-last element on the stack.
 * @see Native TEAL opcode: [`ed25519verify`](https://dev.algorand.co/reference/algorand-teal/opcodes#ed25519verify)
 * Min AVM version: 1
 */
export declare function ed25519verify(a: bytes, b: bytes, c: bytes): boolean;
/**
 * for (data A, signature B, pubkey C) verify the signature of the data against the pubkey => {0 or 1}
 * @see Native TEAL opcode: [`ed25519verify_bare`](https://dev.algorand.co/reference/algorand-teal/opcodes#ed25519verify_bare)
 * Min AVM version: 7
 */
export declare function ed25519verifyBare(a: bytes, b: bytes, c: bytes): boolean;
/**
 * A raised to the Bth power. Fail if A == B == 0 and on overflow
 * @see Native TEAL opcode: [`exp`](https://dev.algorand.co/reference/algorand-teal/opcodes#exp)
 * Min AVM version: 4
 */
export declare function exp(a: uint64, b: uint64): uint64;
/**
 * A raised to the Bth power as a 128-bit result in two uint64s. X is the high 64 bits, Y is the low. Fail if A == B == 0 or if the results exceeds 2^128-1
 * @see Native TEAL opcode: [`expw`](https://dev.algorand.co/reference/algorand-teal/opcodes#expw)
 * Min AVM version: 4
 */
export declare function expw(a: uint64, b: uint64): readonly [uint64, uint64];
/**
 * A uint16 formed from a range of big-endian bytes from A starting at B up to but not including B+2. If B+2 is larger than the array length, the program fails
 * @see Native TEAL opcode: [`extract_uint16`](https://dev.algorand.co/reference/algorand-teal/opcodes#extract_uint16)
 * Min AVM version: 5
 */
export declare function extractUint16(a: bytes, b: uint64): uint64;
/**
 * A uint32 formed from a range of big-endian bytes from A starting at B up to but not including B+4. If B+4 is larger than the array length, the program fails
 * @see Native TEAL opcode: [`extract_uint32`](https://dev.algorand.co/reference/algorand-teal/opcodes#extract_uint32)
 * Min AVM version: 5
 */
export declare function extractUint32(a: bytes, b: uint64): uint64;
/**
 * A uint64 formed from a range of big-endian bytes from A starting at B up to but not including B+8. If B+8 is larger than the array length, the program fails
 * @see Native TEAL opcode: [`extract_uint64`](https://dev.algorand.co/reference/algorand-teal/opcodes#extract_uint64)
 * Min AVM version: 5
 */
export declare function extractUint64(a: bytes, b: uint64): uint64;
/**
 * for (data A, compressed-format signature B, pubkey C) verify the signature of data against the pubkey
 * @see Native TEAL opcode: [`falcon_verify`](https://dev.algorand.co/reference/algorand-teal/opcodes#falcon_verify)
 * Min AVM version: 12
 */
export declare function falconVerify(a: bytes, b: bytes, c: bytes): boolean;
/**
 * ID of the asset or application created in the Ath transaction of the current group
 * `gaids` fails unless the requested transaction created an asset or application and A < GroupIndex.
 * @see Native TEAL opcode: [`gaids`](https://dev.algorand.co/reference/algorand-teal/opcodes#gaids)
 * Min AVM version: 4
 */
export declare function gaid(a: uint64): uint64;
/**
 * Bth bit of (byte-array or integer) A. If B is greater than or equal to the bit length of the value (8*byte length), the program fails
 * see explanation of bit ordering in setbit
 * @see Native TEAL opcode: [`getbit`](https://dev.algorand.co/reference/algorand-teal/opcodes#getbit)
 * Min AVM version: 3
 */
export declare function getBit(a: uint64 | bytes, b: uint64): uint64;
/**
 * Bth byte of A, as an integer. If B is greater than or equal to the array length, the program fails
 * @see Native TEAL opcode: [`getbyte`](https://dev.algorand.co/reference/algorand-teal/opcodes#getbyte)
 * Min AVM version: 3
 */
export declare function getByte(a: bytes, b: uint64): uint64;
/**
 * Get values for inner transaction in the last group submitted
 */
export declare const GITxn: {
    /**
     * 32 byte address
     * Min AVM version: 6
     */
    sender(t: uint64): Account;
    /**
     * microalgos
     * Min AVM version: 6
     */
    fee(t: uint64): uint64;
    /**
     * round number
     * Min AVM version: 6
     */
    firstValid(t: uint64): uint64;
    /**
     * UNIX timestamp of block before txn.FirstValid. Fails if negative
     * Min AVM version: 7
     */
    firstValidTime(t: uint64): uint64;
    /**
     * round number
     * Min AVM version: 6
     */
    lastValid(t: uint64): uint64;
    /**
     * Any data up to 1024 bytes
     * Min AVM version: 6
     */
    note(t: uint64): bytes;
    /**
     * 32 byte lease value
     * Min AVM version: 6
     */
    lease(t: uint64): bytes;
    /**
     * 32 byte address
     * Min AVM version: 6
     */
    receiver(t: uint64): Account;
    /**
     * microalgos
     * Min AVM version: 6
     */
    amount(t: uint64): uint64;
    /**
     * 32 byte address
     * Min AVM version: 6
     */
    closeRemainderTo(t: uint64): Account;
    /**
     * 32 byte address
     * Min AVM version: 6
     */
    votePk(t: uint64): bytes;
    /**
     * 32 byte address
     * Min AVM version: 6
     */
    selectionPk(t: uint64): bytes;
    /**
     * The first round that the participation key is valid.
     * Min AVM version: 6
     */
    voteFirst(t: uint64): uint64;
    /**
     * The last round that the participation key is valid.
     * Min AVM version: 6
     */
    voteLast(t: uint64): uint64;
    /**
     * Dilution for the 2-level participation key
     * Min AVM version: 6
     */
    voteKeyDilution(t: uint64): uint64;
    /**
     * Transaction type as bytes
     * Min AVM version: 6
     */
    type(t: uint64): bytes;
    /**
     * Transaction type as integer
     * Min AVM version: 6
     */
    typeEnum(t: uint64): TransactionType;
    /**
     * Asset ID
     * Min AVM version: 6
     */
    xferAsset(t: uint64): Asset;
    /**
     * value in Asset's units
     * Min AVM version: 6
     */
    assetAmount(t: uint64): uint64;
    /**
     * 32 byte address. Source of assets if Sender is the Asset's Clawback address.
     * Min AVM version: 6
     */
    assetSender(t: uint64): Account;
    /**
     * 32 byte address
     * Min AVM version: 6
     */
    assetReceiver(t: uint64): Account;
    /**
     * 32 byte address
     * Min AVM version: 6
     */
    assetCloseTo(t: uint64): Account;
    /**
     * Position of this transaction within an atomic transaction group. A stand-alone transaction is implicitly element 0 in a group of 1
     * Min AVM version: 6
     */
    groupIndex(t: uint64): uint64;
    /**
     * The computed ID for this transaction. 32 bytes.
     * Min AVM version: 6
     */
    txId(t: uint64): bytes;
    /**
     * ApplicationID from ApplicationCall transaction
     * Min AVM version: 2
     */
    applicationId(t: uint64): Application;
    /**
     * ApplicationCall transaction on completion action
     * Min AVM version: 2
     */
    onCompletion(t: uint64): OnCompleteAction;
    /**
     * Arguments passed to the application in the ApplicationCall transaction
     * Min AVM version: 2
     */
    applicationArgs(t: uint64, a: uint64): bytes;
    /**
     * Number of ApplicationArgs
     * Min AVM version: 2
     */
    numAppArgs(t: uint64): uint64;
    /**
     * Accounts listed in the ApplicationCall transaction
     * Min AVM version: 2
     */
    accounts(t: uint64, a: uint64): Account;
    /**
     * Number of Accounts
     * Min AVM version: 2
     */
    numAccounts(t: uint64): uint64;
    /**
     * Approval program
     * Min AVM version: 2
     */
    approvalProgram(t: uint64): bytes;
    /**
     * Clear state program
     * Min AVM version: 2
     */
    clearStateProgram(t: uint64): bytes;
    /**
     * 32 byte Sender's new AuthAddr
     * Min AVM version: 2
     */
    rekeyTo(t: uint64): Account;
    /**
     * Asset ID in asset config transaction
     * Min AVM version: 2
     */
    configAsset(t: uint64): Asset;
    /**
     * Total number of units of this asset created
     * Min AVM version: 2
     */
    configAssetTotal(t: uint64): uint64;
    /**
     * Number of digits to display after the decimal place when displaying the asset
     * Min AVM version: 2
     */
    configAssetDecimals(t: uint64): uint64;
    /**
     * Whether the asset's slots are frozen by default or not, 0 or 1
     * Min AVM version: 2
     */
    configAssetDefaultFrozen(t: uint64): boolean;
    /**
     * Unit name of the asset
     * Min AVM version: 2
     */
    configAssetUnitName(t: uint64): bytes;
    /**
     * The asset name
     * Min AVM version: 2
     */
    configAssetName(t: uint64): bytes;
    /**
     * URL
     * Min AVM version: 2
     */
    configAssetUrl(t: uint64): bytes;
    /**
     * 32 byte commitment to unspecified asset metadata
     * Min AVM version: 2
     */
    configAssetMetadataHash(t: uint64): bytes;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    configAssetManager(t: uint64): Account;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    configAssetReserve(t: uint64): Account;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    configAssetFreeze(t: uint64): Account;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    configAssetClawback(t: uint64): Account;
    /**
     * Asset ID being frozen or un-frozen
     * Min AVM version: 2
     */
    freezeAsset(t: uint64): Asset;
    /**
     * 32 byte address of the account whose asset slot is being frozen or un-frozen
     * Min AVM version: 2
     */
    freezeAssetAccount(t: uint64): Account;
    /**
     * The new frozen value, 0 or 1
     * Min AVM version: 2
     */
    freezeAssetFrozen(t: uint64): boolean;
    /**
     * Foreign Assets listed in the ApplicationCall transaction
     * Min AVM version: 3
     */
    assets(t: uint64, a: uint64): Asset;
    /**
     * Number of Assets
     * Min AVM version: 3
     */
    numAssets(t: uint64): uint64;
    /**
     * Foreign Apps listed in the ApplicationCall transaction
     * Min AVM version: 3
     */
    applications(t: uint64, a: uint64): Application;
    /**
     * Number of Applications
     * Min AVM version: 3
     */
    numApplications(t: uint64): uint64;
    /**
     * Number of global state integers in ApplicationCall
     * Min AVM version: 3
     */
    globalNumUint(t: uint64): uint64;
    /**
     * Number of global state byteslices in ApplicationCall
     * Min AVM version: 3
     */
    globalNumByteSlice(t: uint64): uint64;
    /**
     * Number of local state integers in ApplicationCall
     * Min AVM version: 3
     */
    localNumUint(t: uint64): uint64;
    /**
     * Number of local state byteslices in ApplicationCall
     * Min AVM version: 3
     */
    localNumByteSlice(t: uint64): uint64;
    /**
     * Number of additional pages for each of the application's approval and clear state programs. An ExtraProgramPages of 1 means 2048 more total bytes, or 1024 for each program.
     * Min AVM version: 4
     */
    extraProgramPages(t: uint64): uint64;
    /**
     * Marks an account nonparticipating for rewards
     * Min AVM version: 5
     */
    nonparticipation(t: uint64): boolean;
    /**
     * Log messages emitted by an application call (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    logs(t: uint64, a: uint64): bytes;
    /**
     * Number of Logs (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    numLogs(t: uint64): uint64;
    /**
     * Asset ID allocated by the creation of an ASA (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    createdAssetId(t: uint64): Asset;
    /**
     * ApplicationID allocated by the creation of an application (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    createdApplicationId(t: uint64): Application;
    /**
     * The last message emitted. Empty bytes if none were emitted. Application mode only
     * Min AVM version: 6
     */
    lastLog(t: uint64): bytes;
    /**
     * 64 byte state proof public key
     * Min AVM version: 6
     */
    stateProofPk(t: uint64): bytes;
    /**
     * Approval Program as an array of pages
     * Min AVM version: 7
     */
    approvalProgramPages(t: uint64, a: uint64): bytes;
    /**
     * Number of Approval Program pages
     * Min AVM version: 7
     */
    numApprovalProgramPages(t: uint64): uint64;
    /**
     * ClearState Program as an array of pages
     * Min AVM version: 7
     */
    clearStateProgramPages(t: uint64, a: uint64): bytes;
    /**
     * Number of ClearState Program pages
     * Min AVM version: 7
     */
    numClearStateProgramPages(t: uint64): uint64;
};
/**
 * Bth scratch space value of the Ath transaction in the current group
 * @see Native TEAL opcode: [`gloadss`](https://dev.algorand.co/reference/algorand-teal/opcodes#gloadss)
 * Min AVM version: 6
 */
export declare function gloadBytes(a: uint64, b: uint64): bytes;
/**
 * Bth scratch space value of the Ath transaction in the current group
 * @see Native TEAL opcode: [`gloadss`](https://dev.algorand.co/reference/algorand-teal/opcodes#gloadss)
 * Min AVM version: 6
 */
export declare function gloadUint64(a: uint64, b: uint64): uint64;
export declare const Global: {
    /**
     * microalgos
     * Min AVM version: 1
     */
    readonly minTxnFee: uint64;
    /**
     * microalgos
     * Min AVM version: 1
     */
    readonly minBalance: uint64;
    /**
     * rounds
     * Min AVM version: 1
     */
    readonly maxTxnLife: uint64;
    /**
     * 32 byte address of all zero bytes
     * Min AVM version: 1
     */
    readonly zeroAddress: Account;
    /**
     * Number of transactions in this atomic transaction group. At least 1
     * Min AVM version: 1
     */
    readonly groupSize: uint64;
    /**
     * Maximum supported version
     * Min AVM version: 2
     */
    readonly logicSigVersion: uint64;
    /**
     * Current round number. Application mode only.
     * Min AVM version: 2
     */
    readonly round: uint64;
    /**
     * Last confirmed block UNIX timestamp. Fails if negative. Application mode only.
     * Min AVM version: 2
     */
    readonly latestTimestamp: uint64;
    /**
     * ID of current application executing. Application mode only.
     * Min AVM version: 2
     */
    readonly currentApplicationId: Application;
    /**
     * Address of the creator of the current application. Application mode only.
     * Min AVM version: 3
     */
    readonly creatorAddress: Account;
    /**
     * Address that the current application controls. Application mode only.
     * Min AVM version: 5
     */
    readonly currentApplicationAddress: Account;
    /**
     * ID of the transaction group. 32 zero bytes if the transaction is not part of a group.
     * Min AVM version: 5
     */
    readonly groupId: bytes;
    /**
     * The remaining cost that can be spent by opcodes in this program.
     * Min AVM version: 6
     */
    readonly opcodeBudget: uint64;
    /**
     * The application ID of the application that called this application. 0 if this application is at the top-level. Application mode only.
     * Min AVM version: 6
     */
    readonly callerApplicationId: uint64;
    /**
     * The application address of the application that called this application. ZeroAddress if this application is at the top-level. Application mode only.
     * Min AVM version: 6
     */
    readonly callerApplicationAddress: Account;
    /**
     * The additional minimum balance required to create (and opt-in to) an asset.
     * Min AVM version: 10
     */
    readonly assetCreateMinBalance: uint64;
    /**
     * The additional minimum balance required to opt-in to an asset.
     * Min AVM version: 10
     */
    readonly assetOptInMinBalance: uint64;
    /**
     * The Genesis Hash for the network.
     * Min AVM version: 10
     */
    readonly genesisHash: bytes;
    /**
     * Whether block proposal payouts are enabled.
     * Min AVM version: 11
     */
    readonly payoutsEnabled: boolean;
    /**
     * The fee required in a keyreg transaction to make an account incentive eligible.
     * Min AVM version: 11
     */
    readonly payoutsGoOnlineFee: uint64;
    /**
     * The percentage of transaction fees in a block that can be paid to the block proposer.
     * Min AVM version: 11
     */
    readonly payoutsPercent: uint64;
    /**
     * The minimum balance an account must have in the agreement round to receive block payouts in the proposal round.
     * Min AVM version: 11
     */
    readonly payoutsMinBalance: uint64;
    /**
     * The maximum balance an account can have in the agreement round to receive block payouts in the proposal round.
     * Min AVM version: 11
     */
    readonly payoutsMaxBalance: uint64;
};
/**
 * Get values for transactions in the current group
 */
export declare const GTxn: {
    /**
     * 32 byte address
     * Min AVM version: 3
     */
    sender(a: uint64): Account;
    /**
     * microalgos
     * Min AVM version: 3
     */
    fee(a: uint64): uint64;
    /**
     * round number
     * Min AVM version: 3
     */
    firstValid(a: uint64): uint64;
    /**
     * UNIX timestamp of block before txn.FirstValid. Fails if negative
     * Min AVM version: 7
     */
    firstValidTime(a: uint64): uint64;
    /**
     * round number
     * Min AVM version: 3
     */
    lastValid(a: uint64): uint64;
    /**
     * Any data up to 1024 bytes
     * Min AVM version: 3
     */
    note(a: uint64): bytes;
    /**
     * 32 byte lease value
     * Min AVM version: 3
     */
    lease(a: uint64): bytes;
    /**
     * 32 byte address
     * Min AVM version: 3
     */
    receiver(a: uint64): Account;
    /**
     * microalgos
     * Min AVM version: 3
     */
    amount(a: uint64): uint64;
    /**
     * 32 byte address
     * Min AVM version: 3
     */
    closeRemainderTo(a: uint64): Account;
    /**
     * 32 byte address
     * Min AVM version: 3
     */
    votePk(a: uint64): bytes;
    /**
     * 32 byte address
     * Min AVM version: 3
     */
    selectionPk(a: uint64): bytes;
    /**
     * The first round that the participation key is valid.
     * Min AVM version: 3
     */
    voteFirst(a: uint64): uint64;
    /**
     * The last round that the participation key is valid.
     * Min AVM version: 3
     */
    voteLast(a: uint64): uint64;
    /**
     * Dilution for the 2-level participation key
     * Min AVM version: 3
     */
    voteKeyDilution(a: uint64): uint64;
    /**
     * Transaction type as bytes
     * Min AVM version: 3
     */
    type(a: uint64): bytes;
    /**
     * Transaction type as integer
     * Min AVM version: 3
     */
    typeEnum(a: uint64): TransactionType;
    /**
     * Asset ID
     * Min AVM version: 3
     */
    xferAsset(a: uint64): Asset;
    /**
     * value in Asset's units
     * Min AVM version: 3
     */
    assetAmount(a: uint64): uint64;
    /**
     * 32 byte address. Source of assets if Sender is the Asset's Clawback address.
     * Min AVM version: 3
     */
    assetSender(a: uint64): Account;
    /**
     * 32 byte address
     * Min AVM version: 3
     */
    assetReceiver(a: uint64): Account;
    /**
     * 32 byte address
     * Min AVM version: 3
     */
    assetCloseTo(a: uint64): Account;
    /**
     * Position of this transaction within an atomic transaction group. A stand-alone transaction is implicitly element 0 in a group of 1
     * Min AVM version: 3
     */
    groupIndex(a: uint64): uint64;
    /**
     * The computed ID for this transaction. 32 bytes.
     * Min AVM version: 3
     */
    txId(a: uint64): bytes;
    /**
     * ApplicationID from ApplicationCall transaction
     * Min AVM version: 2
     */
    applicationId(a: uint64): Application;
    /**
     * ApplicationCall transaction on completion action
     * Min AVM version: 2
     */
    onCompletion(a: uint64): OnCompleteAction;
    /**
     * Arguments passed to the application in the ApplicationCall transaction
     * Min AVM version: 2
     */
    applicationArgs(a: uint64, b: uint64): bytes;
    /**
     * Number of ApplicationArgs
     * Min AVM version: 2
     */
    numAppArgs(a: uint64): uint64;
    /**
     * Accounts listed in the ApplicationCall transaction
     * Min AVM version: 2
     */
    accounts(a: uint64, b: uint64): Account;
    /**
     * Number of Accounts
     * Min AVM version: 2
     */
    numAccounts(a: uint64): uint64;
    /**
     * Approval program
     * Min AVM version: 2
     */
    approvalProgram(a: uint64): bytes;
    /**
     * Clear state program
     * Min AVM version: 2
     */
    clearStateProgram(a: uint64): bytes;
    /**
     * 32 byte Sender's new AuthAddr
     * Min AVM version: 2
     */
    rekeyTo(a: uint64): Account;
    /**
     * Asset ID in asset config transaction
     * Min AVM version: 2
     */
    configAsset(a: uint64): Asset;
    /**
     * Total number of units of this asset created
     * Min AVM version: 2
     */
    configAssetTotal(a: uint64): uint64;
    /**
     * Number of digits to display after the decimal place when displaying the asset
     * Min AVM version: 2
     */
    configAssetDecimals(a: uint64): uint64;
    /**
     * Whether the asset's slots are frozen by default or not, 0 or 1
     * Min AVM version: 2
     */
    configAssetDefaultFrozen(a: uint64): boolean;
    /**
     * Unit name of the asset
     * Min AVM version: 2
     */
    configAssetUnitName(a: uint64): bytes;
    /**
     * The asset name
     * Min AVM version: 2
     */
    configAssetName(a: uint64): bytes;
    /**
     * URL
     * Min AVM version: 2
     */
    configAssetUrl(a: uint64): bytes;
    /**
     * 32 byte commitment to unspecified asset metadata
     * Min AVM version: 2
     */
    configAssetMetadataHash(a: uint64): bytes;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    configAssetManager(a: uint64): Account;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    configAssetReserve(a: uint64): Account;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    configAssetFreeze(a: uint64): Account;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    configAssetClawback(a: uint64): Account;
    /**
     * Asset ID being frozen or un-frozen
     * Min AVM version: 2
     */
    freezeAsset(a: uint64): Asset;
    /**
     * 32 byte address of the account whose asset slot is being frozen or un-frozen
     * Min AVM version: 2
     */
    freezeAssetAccount(a: uint64): Account;
    /**
     * The new frozen value, 0 or 1
     * Min AVM version: 2
     */
    freezeAssetFrozen(a: uint64): boolean;
    /**
     * Foreign Assets listed in the ApplicationCall transaction
     * Min AVM version: 3
     */
    assets(a: uint64, b: uint64): Asset;
    /**
     * Number of Assets
     * Min AVM version: 3
     */
    numAssets(a: uint64): uint64;
    /**
     * Foreign Apps listed in the ApplicationCall transaction
     * Min AVM version: 3
     */
    applications(a: uint64, b: uint64): Application;
    /**
     * Number of Applications
     * Min AVM version: 3
     */
    numApplications(a: uint64): uint64;
    /**
     * Number of global state integers in ApplicationCall
     * Min AVM version: 3
     */
    globalNumUint(a: uint64): uint64;
    /**
     * Number of global state byteslices in ApplicationCall
     * Min AVM version: 3
     */
    globalNumByteSlice(a: uint64): uint64;
    /**
     * Number of local state integers in ApplicationCall
     * Min AVM version: 3
     */
    localNumUint(a: uint64): uint64;
    /**
     * Number of local state byteslices in ApplicationCall
     * Min AVM version: 3
     */
    localNumByteSlice(a: uint64): uint64;
    /**
     * Number of additional pages for each of the application's approval and clear state programs. An ExtraProgramPages of 1 means 2048 more total bytes, or 1024 for each program.
     * Min AVM version: 4
     */
    extraProgramPages(a: uint64): uint64;
    /**
     * Marks an account nonparticipating for rewards
     * Min AVM version: 5
     */
    nonparticipation(a: uint64): boolean;
    /**
     * Log messages emitted by an application call (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    logs(a: uint64, b: uint64): bytes;
    /**
     * Number of Logs (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    numLogs(a: uint64): uint64;
    /**
     * Asset ID allocated by the creation of an ASA (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    createdAssetId(a: uint64): Asset;
    /**
     * ApplicationID allocated by the creation of an application (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    createdApplicationId(a: uint64): Application;
    /**
     * The last message emitted. Empty bytes if none were emitted. Application mode only
     * Min AVM version: 6
     */
    lastLog(a: uint64): bytes;
    /**
     * 64 byte state proof public key
     * Min AVM version: 6
     */
    stateProofPk(a: uint64): bytes;
    /**
     * Approval Program as an array of pages
     * Min AVM version: 7
     */
    approvalProgramPages(a: uint64, b: uint64): bytes;
    /**
     * Number of Approval Program pages
     * Min AVM version: 7
     */
    numApprovalProgramPages(a: uint64): uint64;
    /**
     * ClearState Program as an array of pages
     * Min AVM version: 7
     */
    clearStateProgramPages(a: uint64, b: uint64): bytes;
    /**
     * Number of ClearState Program pages
     * Min AVM version: 7
     */
    numClearStateProgramPages(a: uint64): uint64;
};
/**
 * converts uint64 A to big-endian byte array, always of length 8
 * @see Native TEAL opcode: [`itob`](https://dev.algorand.co/reference/algorand-teal/opcodes#itob)
 * Min AVM version: 1
 */
export declare function itob(a: uint64): bytes;
/**
 * Get values for the last inner transaction
 */
export declare const ITxn: {
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    readonly sender: Account;
    /**
     * microalgos
     * Min AVM version: 5
     */
    readonly fee: uint64;
    /**
     * round number
     * Min AVM version: 5
     */
    readonly firstValid: uint64;
    /**
     * UNIX timestamp of block before txn.FirstValid. Fails if negative
     * Min AVM version: 7
     */
    readonly firstValidTime: uint64;
    /**
     * round number
     * Min AVM version: 5
     */
    readonly lastValid: uint64;
    /**
     * Any data up to 1024 bytes
     * Min AVM version: 5
     */
    readonly note: bytes;
    /**
     * 32 byte lease value
     * Min AVM version: 5
     */
    readonly lease: bytes;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    readonly receiver: Account;
    /**
     * microalgos
     * Min AVM version: 5
     */
    readonly amount: uint64;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    readonly closeRemainderTo: Account;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    readonly votePk: bytes;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    readonly selectionPk: bytes;
    /**
     * The first round that the participation key is valid.
     * Min AVM version: 5
     */
    readonly voteFirst: uint64;
    /**
     * The last round that the participation key is valid.
     * Min AVM version: 5
     */
    readonly voteLast: uint64;
    /**
     * Dilution for the 2-level participation key
     * Min AVM version: 5
     */
    readonly voteKeyDilution: uint64;
    /**
     * Transaction type as bytes
     * Min AVM version: 5
     */
    readonly type: bytes;
    /**
     * Transaction type as integer
     * Min AVM version: 5
     */
    readonly typeEnum: TransactionType;
    /**
     * Asset ID
     * Min AVM version: 5
     */
    readonly xferAsset: Asset;
    /**
     * value in Asset's units
     * Min AVM version: 5
     */
    readonly assetAmount: uint64;
    /**
     * 32 byte address. Source of assets if Sender is the Asset's Clawback address.
     * Min AVM version: 5
     */
    readonly assetSender: Account;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    readonly assetReceiver: Account;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    readonly assetCloseTo: Account;
    /**
     * Position of this transaction within an atomic transaction group. A stand-alone transaction is implicitly element 0 in a group of 1
     * Min AVM version: 5
     */
    readonly groupIndex: uint64;
    /**
     * The computed ID for this transaction. 32 bytes.
     * Min AVM version: 5
     */
    readonly txId: bytes;
    /**
     * ApplicationID from ApplicationCall transaction
     * Min AVM version: 2
     */
    readonly applicationId: Application;
    /**
     * ApplicationCall transaction on completion action
     * Min AVM version: 2
     */
    readonly onCompletion: OnCompleteAction;
    /**
     * Arguments passed to the application in the ApplicationCall transaction
     * Min AVM version: 2
     */
    applicationArgs(a: uint64): bytes;
    /**
     * Number of ApplicationArgs
     * Min AVM version: 2
     */
    readonly numAppArgs: uint64;
    /**
     * Accounts listed in the ApplicationCall transaction
     * Min AVM version: 2
     */
    accounts(a: uint64): Account;
    /**
     * Number of Accounts
     * Min AVM version: 2
     */
    readonly numAccounts: uint64;
    /**
     * Approval program
     * Min AVM version: 2
     */
    readonly approvalProgram: bytes;
    /**
     * Clear state program
     * Min AVM version: 2
     */
    readonly clearStateProgram: bytes;
    /**
     * 32 byte Sender's new AuthAddr
     * Min AVM version: 2
     */
    readonly rekeyTo: Account;
    /**
     * Asset ID in asset config transaction
     * Min AVM version: 2
     */
    readonly configAsset: Asset;
    /**
     * Total number of units of this asset created
     * Min AVM version: 2
     */
    readonly configAssetTotal: uint64;
    /**
     * Number of digits to display after the decimal place when displaying the asset
     * Min AVM version: 2
     */
    readonly configAssetDecimals: uint64;
    /**
     * Whether the asset's slots are frozen by default or not, 0 or 1
     * Min AVM version: 2
     */
    readonly configAssetDefaultFrozen: boolean;
    /**
     * Unit name of the asset
     * Min AVM version: 2
     */
    readonly configAssetUnitName: bytes;
    /**
     * The asset name
     * Min AVM version: 2
     */
    readonly configAssetName: bytes;
    /**
     * URL
     * Min AVM version: 2
     */
    readonly configAssetUrl: bytes;
    /**
     * 32 byte commitment to unspecified asset metadata
     * Min AVM version: 2
     */
    readonly configAssetMetadataHash: bytes;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    readonly configAssetManager: Account;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    readonly configAssetReserve: Account;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    readonly configAssetFreeze: Account;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    readonly configAssetClawback: Account;
    /**
     * Asset ID being frozen or un-frozen
     * Min AVM version: 2
     */
    readonly freezeAsset: Asset;
    /**
     * 32 byte address of the account whose asset slot is being frozen or un-frozen
     * Min AVM version: 2
     */
    readonly freezeAssetAccount: Account;
    /**
     * The new frozen value, 0 or 1
     * Min AVM version: 2
     */
    readonly freezeAssetFrozen: boolean;
    /**
     * Foreign Assets listed in the ApplicationCall transaction
     * Min AVM version: 3
     */
    assets(a: uint64): Asset;
    /**
     * Number of Assets
     * Min AVM version: 3
     */
    readonly numAssets: uint64;
    /**
     * Foreign Apps listed in the ApplicationCall transaction
     * Min AVM version: 3
     */
    applications(a: uint64): Application;
    /**
     * Number of Applications
     * Min AVM version: 3
     */
    readonly numApplications: uint64;
    /**
     * Number of global state integers in ApplicationCall
     * Min AVM version: 3
     */
    readonly globalNumUint: uint64;
    /**
     * Number of global state byteslices in ApplicationCall
     * Min AVM version: 3
     */
    readonly globalNumByteSlice: uint64;
    /**
     * Number of local state integers in ApplicationCall
     * Min AVM version: 3
     */
    readonly localNumUint: uint64;
    /**
     * Number of local state byteslices in ApplicationCall
     * Min AVM version: 3
     */
    readonly localNumByteSlice: uint64;
    /**
     * Number of additional pages for each of the application's approval and clear state programs. An ExtraProgramPages of 1 means 2048 more total bytes, or 1024 for each program.
     * Min AVM version: 4
     */
    readonly extraProgramPages: uint64;
    /**
     * Marks an account nonparticipating for rewards
     * Min AVM version: 5
     */
    readonly nonparticipation: boolean;
    /**
     * Log messages emitted by an application call (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    logs(a: uint64): bytes;
    /**
     * Number of Logs (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    readonly numLogs: uint64;
    /**
     * Asset ID allocated by the creation of an ASA (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    readonly createdAssetId: Asset;
    /**
     * ApplicationID allocated by the creation of an application (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    readonly createdApplicationId: Application;
    /**
     * The last message emitted. Empty bytes if none were emitted. Application mode only
     * Min AVM version: 6
     */
    readonly lastLog: bytes;
    /**
     * 64 byte state proof public key
     * Min AVM version: 6
     */
    readonly stateProofPk: bytes;
    /**
     * Approval Program as an array of pages
     * Min AVM version: 7
     */
    approvalProgramPages(a: uint64): bytes;
    /**
     * Number of Approval Program pages
     * Min AVM version: 7
     */
    readonly numApprovalProgramPages: uint64;
    /**
     * ClearState Program as an array of pages
     * Min AVM version: 7
     */
    clearStateProgramPages(a: uint64): bytes;
    /**
     * Number of ClearState Program pages
     * Min AVM version: 7
     */
    readonly numClearStateProgramPages: uint64;
};
/**
 * Create inner transactions
 */
export declare const ITxnCreate: {
    /**
     * begin preparation of a new inner transaction in a new transaction group
     * `itxn_begin` initializes Sender to the application address; Fee to the minimum allowable, taking into account MinTxnFee and credit from overpaying in earlier transactions; FirstValid/LastValid to the values in the invoking transaction, and all other fields to zero or empty values.
     * @see Native TEAL opcode: [`itxn_begin`](https://dev.algorand.co/reference/algorand-teal/opcodes#itxn_begin)
     * Min AVM version: 5
     */
    begin(): void;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    setSender(a: Account): void;
    /**
     * microalgos
     * Min AVM version: 5
     */
    setFee(a: uint64): void;
    /**
     * Any data up to 1024 bytes
     * Min AVM version: 5
     */
    setNote(a: bytes): void;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    setReceiver(a: Account): void;
    /**
     * microalgos
     * Min AVM version: 5
     */
    setAmount(a: uint64): void;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    setCloseRemainderTo(a: Account): void;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    setVotePk(a: bytes): void;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    setSelectionPk(a: bytes): void;
    /**
     * The first round that the participation key is valid.
     * Min AVM version: 5
     */
    setVoteFirst(a: uint64): void;
    /**
     * The last round that the participation key is valid.
     * Min AVM version: 5
     */
    setVoteLast(a: uint64): void;
    /**
     * Dilution for the 2-level participation key
     * Min AVM version: 5
     */
    setVoteKeyDilution(a: uint64): void;
    /**
     * Transaction type as bytes
     * Min AVM version: 5
     */
    setType(a: bytes): void;
    /**
     * Transaction type as integer
     * Min AVM version: 5
     */
    setTypeEnum(a: uint64): void;
    /**
     * Asset ID
     * Min AVM version: 5
     */
    setXferAsset(a: Asset | uint64): void;
    /**
     * value in Asset's units
     * Min AVM version: 5
     */
    setAssetAmount(a: uint64): void;
    /**
     * 32 byte address. Source of assets if Sender is the Asset's Clawback address.
     * Min AVM version: 5
     */
    setAssetSender(a: Account): void;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    setAssetReceiver(a: Account): void;
    /**
     * 32 byte address
     * Min AVM version: 5
     */
    setAssetCloseTo(a: Account): void;
    /**
     * ApplicationID from ApplicationCall transaction
     * Min AVM version: 2
     */
    setApplicationId(a: Application | uint64): void;
    /**
     * ApplicationCall transaction on completion action
     * Min AVM version: 2
     */
    setOnCompletion(a: uint64): void;
    /**
     * Arguments passed to the application in the ApplicationCall transaction
     * Min AVM version: 2
     */
    setApplicationArgs(a: bytes): void;
    /**
     * Accounts listed in the ApplicationCall transaction
     * Min AVM version: 2
     */
    setAccounts(a: Account): void;
    /**
     * Approval program
     * Min AVM version: 2
     */
    setApprovalProgram(a: bytes): void;
    /**
     * Clear state program
     * Min AVM version: 2
     */
    setClearStateProgram(a: bytes): void;
    /**
     * 32 byte Sender's new AuthAddr
     * Min AVM version: 2
     */
    setRekeyTo(a: Account): void;
    /**
     * Asset ID in asset config transaction
     * Min AVM version: 2
     */
    setConfigAsset(a: Asset | uint64): void;
    /**
     * Total number of units of this asset created
     * Min AVM version: 2
     */
    setConfigAssetTotal(a: uint64): void;
    /**
     * Number of digits to display after the decimal place when displaying the asset
     * Min AVM version: 2
     */
    setConfigAssetDecimals(a: uint64): void;
    /**
     * Whether the asset's slots are frozen by default or not, 0 or 1
     * Min AVM version: 2
     */
    setConfigAssetDefaultFrozen(a: boolean): void;
    /**
     * Unit name of the asset
     * Min AVM version: 2
     */
    setConfigAssetUnitName(a: bytes): void;
    /**
     * The asset name
     * Min AVM version: 2
     */
    setConfigAssetName(a: bytes): void;
    /**
     * URL
     * Min AVM version: 2
     */
    setConfigAssetUrl(a: bytes): void;
    /**
     * 32 byte commitment to unspecified asset metadata
     * Min AVM version: 2
     */
    setConfigAssetMetadataHash(a: bytes): void;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    setConfigAssetManager(a: Account): void;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    setConfigAssetReserve(a: Account): void;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    setConfigAssetFreeze(a: Account): void;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    setConfigAssetClawback(a: Account): void;
    /**
     * Asset ID being frozen or un-frozen
     * Min AVM version: 2
     */
    setFreezeAsset(a: Asset | uint64): void;
    /**
     * 32 byte address of the account whose asset slot is being frozen or un-frozen
     * Min AVM version: 2
     */
    setFreezeAssetAccount(a: Account): void;
    /**
     * The new frozen value, 0 or 1
     * Min AVM version: 2
     */
    setFreezeAssetFrozen(a: boolean): void;
    /**
     * Foreign Assets listed in the ApplicationCall transaction
     * Min AVM version: 3
     */
    setAssets(a: uint64): void;
    /**
     * Foreign Apps listed in the ApplicationCall transaction
     * Min AVM version: 3
     */
    setApplications(a: uint64): void;
    /**
     * Number of global state integers in ApplicationCall
     * Min AVM version: 3
     */
    setGlobalNumUint(a: uint64): void;
    /**
     * Number of global state byteslices in ApplicationCall
     * Min AVM version: 3
     */
    setGlobalNumByteSlice(a: uint64): void;
    /**
     * Number of local state integers in ApplicationCall
     * Min AVM version: 3
     */
    setLocalNumUint(a: uint64): void;
    /**
     * Number of local state byteslices in ApplicationCall
     * Min AVM version: 3
     */
    setLocalNumByteSlice(a: uint64): void;
    /**
     * Number of additional pages for each of the application's approval and clear state programs. An ExtraProgramPages of 1 means 2048 more total bytes, or 1024 for each program.
     * Min AVM version: 4
     */
    setExtraProgramPages(a: uint64): void;
    /**
     * Marks an account nonparticipating for rewards
     * Min AVM version: 5
     */
    setNonparticipation(a: boolean): void;
    /**
     * 64 byte state proof public key
     * Min AVM version: 6
     */
    setStateProofPk(a: bytes): void;
    /**
     * Approval Program as an array of pages
     * Min AVM version: 7
     */
    setApprovalProgramPages(a: bytes): void;
    /**
     * ClearState Program as an array of pages
     * Min AVM version: 7
     */
    setClearStateProgramPages(a: bytes): void;
    /**
     * begin preparation of a new inner transaction in the same transaction group
     * `itxn_next` initializes the transaction exactly as `itxn_begin` does
     * @see Native TEAL opcode: [`itxn_next`](https://dev.algorand.co/reference/algorand-teal/opcodes#itxn_next)
     * Min AVM version: 6
     */
    next(): void;
    /**
     * execute the current inner transaction group. Fail if executing this group would exceed the inner transaction limit, or if any transaction in the group fails.
     * `itxn_submit` resets the current transaction so that it can not be resubmitted. A new `itxn_begin` is required to prepare another inner transaction.
     * @see Native TEAL opcode: [`itxn_submit`](https://dev.algorand.co/reference/algorand-teal/opcodes#itxn_submit)
     * Min AVM version: 5
     */
    submit(): void;
};
export declare const JsonRef: {
    jsonString(a: bytes, b: bytes): bytes;
    jsonUint64(a: bytes, b: bytes): uint64;
    jsonObject(a: bytes, b: bytes): bytes;
};
/**
 * Keccak256 hash of value A, yields [32]byte
 * @see Native TEAL opcode: [`keccak256`](https://dev.algorand.co/reference/algorand-teal/opcodes#keccak256)
 * Min AVM version: 1
 */
export declare function keccak256(a: bytes): bytes;
/**
 * yields length of byte value A
 * @see Native TEAL opcode: [`len`](https://dev.algorand.co/reference/algorand-teal/opcodes#len)
 * Min AVM version: 1
 */
export declare function len(a: bytes): uint64;
/**
 * Load or store scratch values
 */
export declare const Scratch: {
    /**
     * Ath scratch space value.  All scratch spaces are 0 at program start.
     * @see Native TEAL opcode: [`loads`](https://dev.algorand.co/reference/algorand-teal/opcodes#loads)
     * Min AVM version: 5
     */
    loadBytes(a: uint64): bytes;
    /**
     * Ath scratch space value.  All scratch spaces are 0 at program start.
     * @see Native TEAL opcode: [`loads`](https://dev.algorand.co/reference/algorand-teal/opcodes#loads)
     * Min AVM version: 5
     */
    loadUint64(a: uint64): uint64;
    /**
     * store B to the Ath scratch space
     * @see Native TEAL opcode: [`stores`](https://dev.algorand.co/reference/algorand-teal/opcodes#stores)
     * Min AVM version: 5
     */
    store(a: uint64, b: uint64 | bytes): void;
};
/**
 * MiMC hash of scalars A, using curve and parameters specified by configuration C
 * A is a list of concatenated 32 byte big-endian unsigned integer scalars.  Fail if A's length is not a multiple of 32 or any element exceeds the curve modulus.
 * The MiMC hash function has known collisions since any input which is a multiple of the elliptic curve modulus will hash to the same value. MiMC is thus not a general purpose hash function, but meant to be used in zero knowledge applications to match a zk-circuit implementation.
 * @see Native TEAL opcode: [`mimc`](https://dev.algorand.co/reference/algorand-teal/opcodes#mimc)
 * Min AVM version: 11
 */
export declare function mimc(c: MimcConfigurations, a: bytes): bytes;
/**
 * minimum required balance for account A, in microalgos. Required balance is affected by ASA, App, and Box usage. When creating or opting into an app, the minimum balance grows before the app code runs, therefore the increase is visible there. When deleting or closing out, the minimum balance decreases after the app executes. Changes caused by inner transactions or box usage are observable immediately following the opcode effecting the change.
 * @param Txn.Accounts offset (or, since v4, an _available_ account address), _available_ application id (or, since v4, a Txn.ForeignApps offset).
 *  * @return value.
 * @see Native TEAL opcode: [`min_balance`](https://dev.algorand.co/reference/algorand-teal/opcodes#min_balance)
 * Min AVM version: 3
 */
export declare function minBalance(a: Account | uint64): uint64;
/**
 * A times B as a 128-bit result in two uint64s. X is the high 64 bits, Y is the low
 * @see Native TEAL opcode: [`mulw`](https://dev.algorand.co/reference/algorand-teal/opcodes#mulw)
 * Min AVM version: 1
 */
export declare function mulw(a: uint64, b: uint64): readonly [uint64, uint64];
/**
 * the total online stake in the agreement round
 * @see Native TEAL opcode: [`online_stake`](https://dev.algorand.co/reference/algorand-teal/opcodes#online_stake)
 * Min AVM version: 11
 */
export declare function onlineStake(): uint64;
/**
 * Copy of A with the bytes starting at B replaced by the bytes of C. Fails if B+len(C) exceeds len(A)
 * `replace3` can be called using `replace` with no immediates.
 * @see Native TEAL opcode: [`replace3`](https://dev.algorand.co/reference/algorand-teal/opcodes#replace3)
 * Min AVM version: 7
 */
export declare function replace(a: bytes, b: uint64, c: bytes): bytes;
/**
 * Copy of A with the Bth byte set to small integer (between 0..255) C. If B is greater than or equal to the array length, the program fails
 * @see Native TEAL opcode: [`setbyte`](https://dev.algorand.co/reference/algorand-teal/opcodes#setbyte)
 * Min AVM version: 3
 */
export declare function setByte(a: bytes, b: uint64, c: uint64): bytes;
/**
 * SHA256 hash of value A, yields [32]byte
 * @see Native TEAL opcode: [`sha256`](https://dev.algorand.co/reference/algorand-teal/opcodes#sha256)
 * Min AVM version: 1
 */
export declare function sha256(a: bytes): bytes;
/**
 * SHA3_256 hash of value A, yields [32]byte
 * @see Native TEAL opcode: [`sha3_256`](https://dev.algorand.co/reference/algorand-teal/opcodes#sha3_256)
 * Min AVM version: 7
 */
export declare function sha3_256(a: bytes): bytes;
/**
 * SHA512_256 hash of value A, yields [32]byte
 * @see Native TEAL opcode: [`sha512_256`](https://dev.algorand.co/reference/algorand-teal/opcodes#sha512_256)
 * Min AVM version: 1
 */
export declare function sha512_256(a: bytes): bytes;
/**
 * A times 2^B, modulo 2^64
 * @see Native TEAL opcode: [`shl`](https://dev.algorand.co/reference/algorand-teal/opcodes#shl)
 * Min AVM version: 4
 */
export declare function shl(a: uint64, b: uint64): uint64;
/**
 * A divided by 2^B
 * @see Native TEAL opcode: [`shr`](https://dev.algorand.co/reference/algorand-teal/opcodes#shr)
 * Min AVM version: 4
 */
export declare function shr(a: uint64, b: uint64): uint64;
/**
 * The largest integer I such that I^2 <= A
 * @see Native TEAL opcode: [`sqrt`](https://dev.algorand.co/reference/algorand-teal/opcodes#sqrt)
 * Min AVM version: 4
 */
export declare function sqrt(a: uint64): uint64;
/**
 * A range of bytes from A starting at B up to but not including C. If C < B, or either is larger than the array length, the program fails
 * @see Native TEAL opcode: [`substring3`](https://dev.algorand.co/reference/algorand-teal/opcodes#substring3)
 * Min AVM version: 2
 */
export declare function substring(a: bytes, b: uint64, c: uint64): bytes;
/**
 * sumhash512 of value A, yields [64]byte
 * @see Native TEAL opcode: [`sumhash512`](https://dev.algorand.co/reference/algorand-teal/opcodes#sumhash512)
 * Min AVM version: 12
 */
export declare function sumhash512(a: bytes): bytes;
/**
 * Get values for the current executing transaction
 */
export declare const Txn: {
    /**
     * 32 byte address
     * Min AVM version: 1
     */
    readonly sender: Account;
    /**
     * microalgos
     * Min AVM version: 1
     */
    readonly fee: uint64;
    /**
     * round number
     * Min AVM version: 1
     */
    readonly firstValid: uint64;
    /**
     * UNIX timestamp of block before txn.FirstValid. Fails if negative
     * Min AVM version: 7
     */
    readonly firstValidTime: uint64;
    /**
     * round number
     * Min AVM version: 1
     */
    readonly lastValid: uint64;
    /**
     * Any data up to 1024 bytes
     * Min AVM version: 1
     */
    readonly note: bytes;
    /**
     * 32 byte lease value
     * Min AVM version: 1
     */
    readonly lease: bytes;
    /**
     * 32 byte address
     * Min AVM version: 1
     */
    readonly receiver: Account;
    /**
     * microalgos
     * Min AVM version: 1
     */
    readonly amount: uint64;
    /**
     * 32 byte address
     * Min AVM version: 1
     */
    readonly closeRemainderTo: Account;
    /**
     * 32 byte address
     * Min AVM version: 1
     */
    readonly votePk: bytes;
    /**
     * 32 byte address
     * Min AVM version: 1
     */
    readonly selectionPk: bytes;
    /**
     * The first round that the participation key is valid.
     * Min AVM version: 1
     */
    readonly voteFirst: uint64;
    /**
     * The last round that the participation key is valid.
     * Min AVM version: 1
     */
    readonly voteLast: uint64;
    /**
     * Dilution for the 2-level participation key
     * Min AVM version: 1
     */
    readonly voteKeyDilution: uint64;
    /**
     * Transaction type as bytes
     * Min AVM version: 1
     */
    readonly type: bytes;
    /**
     * Transaction type as integer
     * Min AVM version: 1
     */
    readonly typeEnum: TransactionType;
    /**
     * Asset ID
     * Min AVM version: 1
     */
    readonly xferAsset: Asset;
    /**
     * value in Asset's units
     * Min AVM version: 1
     */
    readonly assetAmount: uint64;
    /**
     * 32 byte address. Source of assets if Sender is the Asset's Clawback address.
     * Min AVM version: 1
     */
    readonly assetSender: Account;
    /**
     * 32 byte address
     * Min AVM version: 1
     */
    readonly assetReceiver: Account;
    /**
     * 32 byte address
     * Min AVM version: 1
     */
    readonly assetCloseTo: Account;
    /**
     * Position of this transaction within an atomic transaction group. A stand-alone transaction is implicitly element 0 in a group of 1
     * Min AVM version: 1
     */
    readonly groupIndex: uint64;
    /**
     * The computed ID for this transaction. 32 bytes.
     * Min AVM version: 1
     */
    readonly txId: bytes;
    /**
     * ApplicationID from ApplicationCall transaction
     * Min AVM version: 2
     */
    readonly applicationId: Application;
    /**
     * ApplicationCall transaction on completion action
     * Min AVM version: 2
     */
    readonly onCompletion: OnCompleteAction;
    /**
     * Arguments passed to the application in the ApplicationCall transaction
     * Min AVM version: 2
     */
    applicationArgs(a: uint64): bytes;
    /**
     * Number of ApplicationArgs
     * Min AVM version: 2
     */
    readonly numAppArgs: uint64;
    /**
     * Accounts listed in the ApplicationCall transaction
     * Min AVM version: 2
     */
    accounts(a: uint64): Account;
    /**
     * Number of Accounts
     * Min AVM version: 2
     */
    readonly numAccounts: uint64;
    /**
     * Approval program
     * Min AVM version: 2
     */
    readonly approvalProgram: bytes;
    /**
     * Clear state program
     * Min AVM version: 2
     */
    readonly clearStateProgram: bytes;
    /**
     * 32 byte Sender's new AuthAddr
     * Min AVM version: 2
     */
    readonly rekeyTo: Account;
    /**
     * Asset ID in asset config transaction
     * Min AVM version: 2
     */
    readonly configAsset: Asset;
    /**
     * Total number of units of this asset created
     * Min AVM version: 2
     */
    readonly configAssetTotal: uint64;
    /**
     * Number of digits to display after the decimal place when displaying the asset
     * Min AVM version: 2
     */
    readonly configAssetDecimals: uint64;
    /**
     * Whether the asset's slots are frozen by default or not, 0 or 1
     * Min AVM version: 2
     */
    readonly configAssetDefaultFrozen: boolean;
    /**
     * Unit name of the asset
     * Min AVM version: 2
     */
    readonly configAssetUnitName: bytes;
    /**
     * The asset name
     * Min AVM version: 2
     */
    readonly configAssetName: bytes;
    /**
     * URL
     * Min AVM version: 2
     */
    readonly configAssetUrl: bytes;
    /**
     * 32 byte commitment to unspecified asset metadata
     * Min AVM version: 2
     */
    readonly configAssetMetadataHash: bytes;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    readonly configAssetManager: Account;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    readonly configAssetReserve: Account;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    readonly configAssetFreeze: Account;
    /**
     * 32 byte address
     * Min AVM version: 2
     */
    readonly configAssetClawback: Account;
    /**
     * Asset ID being frozen or un-frozen
     * Min AVM version: 2
     */
    readonly freezeAsset: Asset;
    /**
     * 32 byte address of the account whose asset slot is being frozen or un-frozen
     * Min AVM version: 2
     */
    readonly freezeAssetAccount: Account;
    /**
     * The new frozen value, 0 or 1
     * Min AVM version: 2
     */
    readonly freezeAssetFrozen: boolean;
    /**
     * Foreign Assets listed in the ApplicationCall transaction
     * Min AVM version: 3
     */
    assets(a: uint64): Asset;
    /**
     * Number of Assets
     * Min AVM version: 3
     */
    readonly numAssets: uint64;
    /**
     * Foreign Apps listed in the ApplicationCall transaction
     * Min AVM version: 3
     */
    applications(a: uint64): Application;
    /**
     * Number of Applications
     * Min AVM version: 3
     */
    readonly numApplications: uint64;
    /**
     * Number of global state integers in ApplicationCall
     * Min AVM version: 3
     */
    readonly globalNumUint: uint64;
    /**
     * Number of global state byteslices in ApplicationCall
     * Min AVM version: 3
     */
    readonly globalNumByteSlice: uint64;
    /**
     * Number of local state integers in ApplicationCall
     * Min AVM version: 3
     */
    readonly localNumUint: uint64;
    /**
     * Number of local state byteslices in ApplicationCall
     * Min AVM version: 3
     */
    readonly localNumByteSlice: uint64;
    /**
     * Number of additional pages for each of the application's approval and clear state programs. An ExtraProgramPages of 1 means 2048 more total bytes, or 1024 for each program.
     * Min AVM version: 4
     */
    readonly extraProgramPages: uint64;
    /**
     * Marks an account nonparticipating for rewards
     * Min AVM version: 5
     */
    readonly nonparticipation: boolean;
    /**
     * Log messages emitted by an application call (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    logs(a: uint64): bytes;
    /**
     * Number of Logs (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    readonly numLogs: uint64;
    /**
     * Asset ID allocated by the creation of an ASA (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    readonly createdAssetId: Asset;
    /**
     * ApplicationID allocated by the creation of an application (only with `itxn` in v5). Application mode only
     * Min AVM version: 5
     */
    readonly createdApplicationId: Application;
    /**
     * The last message emitted. Empty bytes if none were emitted. Application mode only
     * Min AVM version: 6
     */
    readonly lastLog: bytes;
    /**
     * 64 byte state proof public key
     * Min AVM version: 6
     */
    readonly stateProofPk: bytes;
    /**
     * Approval Program as an array of pages
     * Min AVM version: 7
     */
    approvalProgramPages(a: uint64): bytes;
    /**
     * Number of Approval Program pages
     * Min AVM version: 7
     */
    readonly numApprovalProgramPages: uint64;
    /**
     * ClearState Program as an array of pages
     * Min AVM version: 7
     */
    clearStateProgramPages(a: uint64): bytes;
    /**
     * Number of ClearState Program pages
     * Min AVM version: 7
     */
    readonly numClearStateProgramPages: uint64;
};
export declare const VoterParams: {
    /**
     * Online stake in microalgos
     * Min AVM version: 11
     */
    voterBalance(a: Account | uint64): readonly [uint64, boolean];
    /**
     * Had this account opted into block payouts
     * Min AVM version: 11
     */
    voterIncentiveEligible(a: Account | uint64): readonly [boolean, boolean];
};
/**
 * Verify the proof B of message A against pubkey C. Returns vrf output and verification flag.
 * `VrfAlgorand` is the VRF used in Algorand. It is ECVRF-ED25519-SHA512-Elligator2, specified in the IETF internet draft [draft-irtf-cfrg-vrf-03](https://datatracker.ietf.org/doc/draft-irtf-cfrg-vrf/03/).
 * @see Native TEAL opcode: [`vrf_verify`](https://dev.algorand.co/reference/algorand-teal/opcodes#vrf_verify)
 * Min AVM version: 7
 */
export declare function vrfVerify(s: VrfVerify, a: bytes, b: bytes, c: bytes): readonly [bytes, boolean];
/**
 * A range of bytes from A starting at B up to the end of the sequence
 */
export declare function extract(a: bytes, b: uint64): bytes;
/**
 * A range of bytes from A starting at B up to but not including B+C. If B+C is larger than the array length, the program fails
 */
export declare function extract(a: bytes, b: uint64, c: uint64): bytes;
/**
 * selects one of two values based on top-of-stack: B if C != 0, else A
 */
export declare function select(a: bytes, b: bytes, c: uint64): bytes;
/**
 * selects one of two values based on top-of-stack: B if C != 0, else A
 */
export declare function select(a: uint64, b: uint64, c: uint64): uint64;
/**
 * Set the nth bit of target to the value of c (1 or 0)
 */
export declare function setBit(target: bytes, n: uint64, c: uint64): bytes;
/**
 * Set the nth bit of target to the value of c (1 or 0)
 */
export declare function setBit(target: uint64, n: uint64, c: uint64): uint64;
